# -*- coding: utf-8 -*-

"""Main module."""

from math import pow, e

def abv(originalGravity: float, finalGravity: float) -> float:
    """
    Determines the Alochol by Volume (ABV).

    Parameters
    ----------
    originalGravity : float
        Original gravity of wort.
    finalGravity : float
        Final gravity of beer.
    
    Returns
    -------
    float or None
        Alcohol by Volume.
    """

    if (originalGravity > finalGravity):
        return ((76.08 * (originalGravity - finalGravity) / (1.775 - originalGravity)) * (finalGravity / 0.794))
    else:
        return

def abw(originalGravity: float, finalGravity: float) -> float:
    """
    Determines Alochol by Weight (ABW).

    Parameters
    ----------
    originalGravity : float
        Original gravity of wort.
    finalGravity : float
        Final gravity of beer.
    
    Returns
    -------
    float or None
        Alcohol by Weight.
    """

    if (originalGravity > finalGravity):
        return ((0.79 * abv(originalGravity, finalGravity)) / finalGravity)
    else:
        return

def mcu(pounds: float, lovibond: float, gallons: float) -> float:
    """
    Determines Malt Color Unit (MCU).

    Parameters
    ----------
    pounds : float
        Weight in pounds.
    lovibond : float
        Color of malt in degrees lovibond.
    gallons : float
        Water volume in gallons.

    Returns
    -------
    float or None
        Malt Color Unit.
    """

    if (gallons != 0):
        return ((pounds * lovibond) / gallons)
    else:
        return


def srm(pounds: float, lovibond: float, gallons: float) -> float:
    """
    Determines Standard Reference Method (SRM).

    Parameters
    ----------
    pounds : float
        Weight in pounds.
    lovibond : float
        Color of malt in degrees lovibond.
    gallons : float
        Water volume in gallons.

    Returns
    -------
    float or None
        Standard Reference Method.
    """
    
    if (gallons != 0):
        return (1.4922 * (pow(mcu(pounds, lovibond, gallons), 0.6859)))
    else:
        return

def aau(alphaAcid: float, ounces: float) -> float:
    """
    Determines Alpha Acid Unit.

    Parameters
    ----------
    alphaAcid : float
        Alpha acid percentage.
    ounces : float
        Weight in ounces.
    
    Returns
    -------
    float or None
        Alpha Acid Unit.
    """

    if (alphaAcid and ounces) > 0:
        return (alphaAcid * ounces)
    else:
        return

def utilization(minutes: float, specificGravity: float) -> float:
    """
    Determines Hop Utilization.

    Parameters
    ----------
    minutes : float
        Time in minutes.
    specificGravity : float
        Specific gravity.
    
    Returns
    -------
    float
        Hop Utilization.
    """

    return ((1.65 * pow(0.000125, (specificGravity - 1))) * (1 - pow(e, (-0.04 * minutes))) / 4.15)

def ibu(alphaAcid: float, ounces: float, minutes: float, specificGravity: float, gallons: float) -> float:
    """
    Determines International Bitterness Units (IBU).

    Parameters
    ----------
    alphaAcid : float
        Alpha acid percentage.
    ounces : float
        Weight in ounces.
    minutes : float
        Time in minutes.
    specificGravity : float
        Specific gravity.
    gallons : float
        Water volume in gallons.

    Returns
    -------
    float or None
        International Bitterness Units.
    """

    if (gallons != 0):
        return (aau(alphaAcid, ounces) * utilization(minutes, specificGravity) * 75 / gallons)
    else:
        return

def convertToPlato(specificGravity: float) -> float:
    """
    Convert Specific Gravity to Plato.

    http://hbd.org/ensmingr/

    Parameters
    ----------
    specificGravity : float
        Specific gravity.
    
    Returns
    -------
    float
        Degress plato.
    """

    return ((-463.37) + (668.72 * specificGravity) - (205.35 * pow(specificGravity, 2)))

def realExtract(originalGravity: float, finalGravity: float) -> float:
    """
    Determine the Real Extract value of wort.

    http://hbd.org/ensmingr/

    Parameters
    ----------
    originalGravity : float
        Original gravity of wort.
    finalGravity : float
        Final gravity of beer.
    
    Returns
    -------
    float or None
        Real extract value.
    """
    
    if (originalGravity > finalGravity):
        return ((0.1808 * convertToPlato(originalGravity)) + (0.8192 * convertToPlato(finalGravity)))
    else:
        return

def calories(originalGravity: float, finalGravity: float) -> float:
    """
    Determine Calories per 12oz Serving.
    
    Parameters
    ----------
    originalGravity : float
        Original gravity of wort.
    finalGravity : float
        Final gravity of beer.
    
    Returns
    -------
    float or None
        Calories.
    """

    if (originalGravity > finalGravity):
        return (((6.9 * abw(originalGravity, finalGravity)) + 4.0 * (realExtract(originalGravity, finalGravity) - 0.1)) * finalGravity * 3.55)
    else:
        return

def attenuation(originalGravity: float, finalGravity: float) -> float:
    """
    Determines Attenuation of yeast.

    Parameters
    ----------
    originalGravity : float
        Original gravity of wort.
    finalGravity : float
        Final gravity of beer.
    
    Returns
    -------
    float or None
        Attenuation.
    """

    if (originalGravity > finalGravity):
        return ((originalGravity - finalGravity) / (originalGravity - 1))
    else:
        return

def gravityCorrection(fahrenheit: float, specificGravity: float, calibrationFahrenheit: float) -> float:
    """
    Determines Gravity Temperature Correction.

    Parameters
    ----------
    fahrenheit : float
        Temperature in fahrenheit.
    specificGravity : float
        Specific gravity.
    calibrationFahrenheit : float
        Calibration temperature in fahrenheit
    
    Returns
    -------
    float
        Gravity temperature correction.
    """

    return (specificGravity * ((1.00130346 - 0.000134722124 * fahrenheit + 0.00000204052596 * pow(fahrenheit, 2) - 0.00000000232820948 * pow(fahrenheit, 3)) / (1.00130346 - 0.000134722124 * calibrationFahrenheit + 0.00000204052596 * pow(calibrationFahrenheit, 2) - 0.00000000232820948 * pow(calibrationFahrenheit, 3))))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `brewerwall` package."""


import unittest
from click.testing import CliRunner

from brewerwall import brewerwall
from brewerwall import cli


class TestBrewerwall(unittest.TestCase):
    """Tests for `brewerwall` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def testABV(self):
        """Test Alcohol by Volume calculation."""
        self.assertEqual(7.319479429051208, brewerwall.abv(1.055, 1))
        self.assertEqual(None, brewerwall.abv(1, 1.055))
    
    def testABW(self):
        """Test Alochol by Weight calculation."""
        self.assertEqual(5.782388748950455, brewerwall.abw(1.055, 1))
        self.assertEqual(None, brewerwall.abw(1, 1.055))
    
    def testMCU(self):
        """Test Malt Color Unit calculation."""
        self.assertEqual(None, brewerwall.mcu(1, 1, 0))
        self.assertEqual(5, brewerwall.mcu(5, 5, 5))
        self.assertEqual(5.5, brewerwall.mcu(5.5, 5.5, 5.5))
    
    def testSRM(self):
        """Test Standard Reference Method calculation."""
        self.assertEqual(None, brewerwall.srm(7, 5, 0))
        self.assertEqual(5.668651803424155, brewerwall.srm(7, 5, 5))
        self.assertEqual(5.943353419684101, brewerwall.srm(7.5, 5.5, 5.5))
    
    def testAAU(self):
        """Test Alpha Acid Unit calculation.

        Based off Palmer's Calculation
        """
        self.assertEqual(9.600000000000001, brewerwall.aau(1.5, 6.4))
        self.assertEqual(4.6, brewerwall.aau(1, 4.6))
    
    def testUtilization(self):
        """Test hop utilization calculation."""
        self.assertEqual(0.08363227080582435, brewerwall.utilization(10, 1.050))
        self.assertEqual(0.30113013986478654, brewerwall.utilization(120, 1.030))
        self.assertEqual(0.0, brewerwall.utilization(0, 1.070))
        self.assertEqual(0.14780486892282785, brewerwall.utilization(45, 1.090))
    
    def testIBU(self):
        """Test International Bittering Units calculation.
        
        Based off Palmer's Calculation
        """
        self.assertEqual(25.365869680614512, brewerwall.ibu(6.4, 1.5, 60, 1.080, 5))
        self.assertEqual(6.03108750923272, brewerwall.ibu(4.6, 1, 15, 1.080, 5))
    
    def testPlato(self):
        """Test conversion from specific gravity to plato."""
        self.assertEqual(17.055185000000108, brewerwall.convertToPlato(1.070))
    
    def testRealExtract(self):
        """Test Real Extract calculation."""
        self.assertEqual(6.216277095999994, brewerwall.realExtract(1.070, 1.015))
        self.assertEqual(None, brewerwall.realExtract(1.015, 1.070))
    
    def testCalories(self):
        """Test calories per 12 oz. serving calculation.
        
        Based on http://hbd.org/ensmingr/
        """
        self.assertEqual(234.97692128247783, brewerwall.calories(1.070, 1.015))
        self.assertEqual(None, brewerwall.calories(1.015, 1.070))
    
    def testAttenuation(self):
        """Test attenuation calculation."""
        self.assertEqual(0.7777777777777778, brewerwall.attenuation(1.054, 1.012))
        self.assertEqual(None, brewerwall.attenuation(1, 1.055))
    
    def testGravityCorrection(self):
        """Test gravity correction calculation."""
        self.assertEqual(1.0562227410996516, brewerwall.gravityCorrection(100.4, 1.050, 60))
    
    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'brewerwall.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output
